/*
 * Expresso, a C++ Array template class library
 * Copyright (C) 1998 Oskar Enoksson (enok@lysator.liu.se)
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */

//
// File purpose: Example of using Ix<N> objects
//

#include "expresso/expresso.h"

using namespace expresso;

int main() {

  // An Ix<N> objext is often used when manipulating Array objects
  // Ix<N> is a simple one dimensional array of N integers.
  // The following constructors exist:

  Ix<2> i1;        // Creates a rank 2 index with undefined initial values
  Ix<3> i2(1);     // Creates a rank 3 index with all values set to 1
  Ix<3> i3(1,2,3); // Creates a rank 3 index with specified initial values
  Ix<3> i4(i3);    // Copy constructor

  // The elements of an index object are manipulated with operator []
   
  i1[0]=10;        // Sets element 0 to value 10
  i2[1]=i1[0];     // Copies element 0 of i3 to element 1 of i2

  // The index objects are copied using operator =
   
  i2=i3;           // Copy all elements in i3 to i2
  i2=24;           // Copy 24 to all elements in i2

  // The elements of a const Ix<N> object cannot be modified

  const Ix<3> i5(0);
  i4=i5;            // Ok
  // i5=i4          // Error!!!

  ///// End Ix  example /////
}
