/* -*- c++ -*- */
/*
 * Expresso, a C++ Array template class library
 * Copyright (C) 1998,2004 Oskar Enoksson (enok@lysator.liu.se)
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */
#ifndef EXPRESSO_STREAM_HDR
#define EXPRESSO_STREAM_HDR

#include <iostream>

namespace esso {

  template<int K>
  class DoStream {
  public:
    template<int N, class Type>
    static void DumpArray(std::ostream &stream, 
			  Array<N,Type> &src, 
			  const Ix<N> &len) {
      for(IXTYPE i=0; i<len[K-1]; i++) {
        DoStream<K-1>::DumpArray(stream,src,len);
	src.Step(K-1);
      }
      src.Step(K-1,-len[K-1]);
    }
    template<int N, class Type>
    static void ReadArray(std::istream &stream, 
			  Array<N,Type> &dst, 
			  const Ix<N> &len) {
      for(IXTYPE i=0; i<len[K-1]; i++) {
        DoStream<K-1>::ReadArray(stream,dst,len);
	dst.Step(K-1);
      }
      dst.Step(K-1,-len[K-1]);
    }
    template<int N, class Type>
    static void DumpArrayBin(std::ostream &stream, 
			     Array<N,Type> &src, 
			     const Ix<N> &len) {
      for(IXTYPE i=0; i<len[K-1]; i++) {
        DoStream<K-1>::DumpArrayBin(stream,src,len);
	src.Step(K-1);
      }
      src.Step(K-1,-len[K-1]);
    }
    template<int N, class Type>
    static void ReadArrayBin(std::istream &stream, 
			     Array<N,Type> &dst, 
			     const Ix<N> &len) {
      for(IXTYPE i=0; i<len[K-1]; i++) {
        DoStream<K-1>::ReadArrayBin(stream,dst,len);
	dst.Step(K-1);
      }
      dst.Step(K-1,-len[K-1]);
    }
  };
  template<>
  class DoStream<1> {
  public:
    template<int N, class Type>
    static void DumpArray(std::ostream &stream, 
			  Array<N,Type> &src, 
			  const Ix<N> &len) {
      for(IXTYPE i=0; i<len[0]; i++)
        stream<<src.CGet(i)<<' ';
      stream<<std::endl;
    }
    template<int N, class Type>
    static void ReadArray(std::istream &stream, 
			  Array<N,Type> &src, 
			  const Ix<N> &len) {
      for(IXTYPE i=0; i<len[0]; i++)
        stream>>src.Get(i);
    }
    template<int N, class Type>
    static void DumpArrayBin(std::ostream &stream, 
			     Array<N,Type> &src, 
			     const Ix<N> &len) {
      for(IXTYPE i=0; i<len[0]; i++)
        stream.write(reinterpret_cast<const char *>(&src.Get(i)),
		     sizeof(typename Array<N,Type>::T)/sizeof(char));
    }
    template<int N, class Type>
    static void ReadArrayBin(std::istream &stream, 
			     Array<N,Type> &src, 
			     const Ix<N> &len) {
      for(IXTYPE i=0; i<len[0]; i++)
        stream.read(reinterpret_cast<char *>(&src.Get(i)),
		    sizeof(typename Array<N,Type>::T)/sizeof(char));
    }
  };

  ///@name Stream functions
  //@{
  /** Unformatted stream store
   *
   * Simply dumps all elements into the \a stream.
   * The dimensions are traversed in increasing order, so that the 
   * outermost loop is over dimension \c N-1.
   *@arg \a stream output stream
   *@arg \a src Source array
   *@return Reference to output stream \a stream
   */
  template<int N, class Type>
  std::ostream & WriteUfmt(std::ostream &stream, const Array<N,Type> &src) {
    Ix<N> len;
    typename Array<N,Type>::IteratorType srcit(src);
    BeginAssign(srcit,len);
    DoStream<N>::DumpArrayBin(stream,srcit,len);
    return stream;
  }
  /** Formatted stream store
   *
   * Simply dumps all elements into the \a stream.
   * The dimensions are traversed in increasing order, so that the 
   * outermost loop is over dimension \c N-1.
   *@arg \a stream output stream
   *@arg \a src Source array
   *@return Reference to output stream \a stream
   */
  template<int N, class Type>
  std::ostream & WriteFmt(std::ostream &stream, const Array<N,Type> &src) {
    Ix<N> len;
    typename Array<N,Type>::IteratorType srcit(src);
    BeginAssign(srcit,len);
    DoStream<N>::DumpArray(stream,srcit,len);
    return stream;
  }
  /** Unformatted stream read
   *
   * Simply reads all elements from the \a stream.
   * The dimensions are traversed in increasing order, so that the 
   * outermost loop is over dimension \c N-1.
   *@arg \a stream output stream
   *@arg \a src Source array
   *@return Reference to input stream \a stream
   */
  template<int N, class Type>
  std::istream & ReadUfmt(std::istream &stream, const Array<N,Type> &dst) {
    Ix<N> len;
    
    typename Array<N,Type>::IteratorType dstit(dst);
    BeginAssign(dstit,len);
    DoStream<N>::ReadArrayBin(stream,dstit,len);
    return stream;
  }
  /** Formatted stream read
   *
   * Simply reads all elements from the \a stream.
   * The dimensions are traversed in increasing order, so that the 
   * outermost loop is over dimension \c N-1.
   *@arg \a stream output stream
   *@arg \a src Source array
   *@return Reference to input stream \a stream
   */
  template<int N, class Type>
  std::istream & ReadFmt(std::istream &stream, const Array<N,Type> &dst) {
    Ix<N> len;
    
    typename Array<N,Type>::IteratorType dstit(dst);
    BeginAssign(dstit,len);
    DoStream<N>::ReadArray(stream,dstit,len);
    return stream;
  }
  /** Formated read of Ix object */
  template<int N, class T>
  std::istream & ReadFmt(std::istream &stream, Ix<N,T> &dst) {
    for(int n=0; n<N; n++)
      stream >>dst[n];
    return stream;
  }
  /** Unformated read of Ix object */
  template<int N, class T>
  std::istream & ReadUfmt(std::istream &stream, Ix<N,T> &dst) {
    stream.read(reinterpret_cast<char *>(&dst),sizeof(dst));
    return stream;
  }
  /** Formatted write of Ix object */
  template<int N, class T>
  std::ostream & WriteFmt(std::ostream &stream, const Ix<N,T> &src) {
    for(int n=0; n<N; n++)
      stream <<src[n]<<' ';
    return stream;
  }
  /** Unformated write of Ix object */
  template<int N, class T>
  std::ostream & WriteUfmt(std::ostream &stream, const Ix<N,T> &src) {
    stream.write(reinterpret_cast<const char *>(&src),sizeof(src));
    return stream;
  }
  /** Ascii stream store operation
   *
   * Output format is
   *\li N
   *\li LEN(0) L(0)
   *\li LEN(1) L(1)
   *\li ...
   *\li LEN(N-1) L(N-1)
   *\li DATA (LINE 1)
   *\li DATA (LINE 2)
   *\li ...
   *
   * where \c N is the array rank, \c LEN(K) is the length along dimension \c K
   * and \c L(K) is the first index in dimension \c K. Each line in \c DATA
   * is along dimension 0, and the other dimensions are traversed
   * in increasing order, so that the outermost loop is over dimension \c N-1.
   *@arg \a stream output stream
   *@arg \a src Source array
   *@return Reference to output stream \a stream
   */
  template<int N, class Type>
  std::ostream & operator <<(std::ostream &stream, const Array<N,Type> &src) {
    stream<<N<<std::endl;
    for(int n=0; n<N; n++) 
      stream<<src.Len(n)<<' '<<src.L(n)<<std::endl;
    return WriteFmt(stream,src);
  }
  /** Ascii stream read operation
   *
   * The elements of the array are read from a file that is 
   * assumed to have the format as written by operator <<
   *
   * If the array index bounds do not match the array found
   * in the file the array elements are unchanged and the fail
   * flag is set for the stream object.
   *@arg \a stream input stream
   *@arg \a src Destination array
   *@return true on success, or false if operation failed
   *@return Reference to input stream \a stream
   */
  template<int N, class Type>
  std::istream & operator >>(std::istream &stream, const Array<N,Type> &dst) {
    do {
      int n;
      stream>>n;
      if(n!=N)
	break;

      Ix<N> len,l;
      for(n=0; n<N; n++) {
	stream>>len[n];
	if(len[n] != dst.L(n)-dst.U(n))
	  break;
	stream>>l[n];
	if(l[n] != dst.L(n))
	  break;
      }
      if(n!=N)
	break;
      return ReadFmt(stream,dst);
    } while(0);

    // If we get here we failed
    stream.setstate(std::istream::failbit);
    return stream;
  }
  /** Realloc Array from contents of formatted stream and read
   * all elements from the stream.
   *
   *@arg \a stream input stream
   *@arg \a dst Destination array
   *@return true on success, or false if operation failed
   *@see Realloc(std::istream &, Array<N,Type> &)
   *@see operator ><(std::istream &, const Array<N,Type> &)
   */
  template<int N, class Type>
  bool Clone(std::istream &stream, Array<N,Type> &dst) {
    int n;
    stream>>n;
    if(n!=N)
      return false;
    
    Ix<N> len,l;
    for(n=0; n<N; n++) {
      stream>>len[n];
      stream>>l[n];
    }
    if(stream.fail())
      return false;
    if(!dst.Realloc(len,l))
      return false;
    return !ReadFmt(stream,dst).fail();
  }
}

#endif
